/* SemWare's TSE editor V2.5 macros to insert escape codes into
   ascii text for a Diconix InkJet Printer Model 150. To make this
   SAL macro operational, invoke the main menu (F10), choose
   "Macro", choose "Compile" and press Enter at "Execute Macro".

....................................................................
Prof. Timo Salmi   Co-moderator of news:comp.archives.msdos.announce
Moderating at ftp:// & http://garbo.uwasa.fi archives  193.166.120.5
Department of Accounting and Business Finance  ; University of Vaasa
ts@uwasa.fi http://uwasa.fi/~ts BBS 961-3170972; FIN-65101,  Finland
*/

// The contents of a simple help, tied later to the CtrlAlt-H key
helpdef tHelpData
  title = "DICONIX.S HELP"        // The help's caption
  x = 10                          // Location
  y = 3
  // The actual help text
  " Prof. Timo Salmi's Diconix Escape Codes"
  ""
  "Inserts escape codes into ascii text for "
  "a Diconix InkJet Printer Model 150 "
  ""
  " You can use <F11> to invoke the command "
  " menu after first exiting this help. "
  ""
  " Last updated Tue 23-January-1996 21:26:05 "
end tHelpData

proc tReset()
  InsertText ("@",_INSERT_)
end

proc tUnderlineOn()
  InsertText ("-1",_INSERT_)
end

proc tUnderlineOff()
  InsertText ("-0",_INSERT_)
end

proc tOverscoreOn()
  InsertText ("_1",_INSERT_)
end

proc tOverscoreOff()
  InsertText ("_0",_INSERT_)
end

proc tSuperScriptOn()
  InsertText ("S0",_INSERT_)
end

proc tSubScriptOn()
  InsertText ("S1",_INSERT_)
end

proc tSuperScriptOff()
  InsertText ("T",_INSERT_)
end

proc tDoubleStrikeOn()
  InsertText ("G",_INSERT_)
end

proc tDoubleStrikeOff()
  InsertText ("H",_INSERT_)
end

proc tEmphasizedOn()
  InsertText ("E",_INSERT_)
end

proc tEmphasizedOff()
  InsertText ("F",_INSERT_)
end

proc tEliteNormal()
  InsertText ("!",_INSERT_)
end

proc tEliteCondensed()
  InsertText ("!",_INSERT_)
end

proc tEliteCondensedEmphasized()
  if CurrPos() <> 1
    Warn("Can only be at the beginning of the line")
  else
    InsertText ("!",_INSERT_)
  endif
end

proc tWide()
  InsertText ("!$",_INSERT_)
end

proc tWideEmphasized()
  InsertText ("!,",_INSERT_)
end

proc tEnlargedOn()
  InsertText ("W1",_INSERT_)
end

proc tEnlargedOff()
  InsertText ("W0",_INSERT_)
end

// Cancel escape sequences all through the file
proc tCancelAll()
  lReplace("@","","gn")
  lReplace("-1","","gn")
  lReplace("-0","","gn")
  lReplace("_1","","gn")
  lReplace("_0","","gn")
  lReplace("E","","gn")
  lReplace("F","","gn")
  lReplace("S0","","gn")
  lReplace("S1","","gn")
  lReplace("T","","gn")
  lReplace("G","","gn")
  lReplace("H","","gn")
  lReplace("!","","gn")
  lReplace("!","","gn")
  lReplace("!","","gn")
  lReplace("!$","","gn")
  lReplace("!,","","gn")
  lReplace("W1","","gn")
  lReplace("W0","","gn")
end tCancelAll

// Cancel escape sequences on the current line
proc tCancelLine()
  PushPosition()
  BegLine()
  lReplace("@","","cn")
  lReplace("-1","","cn")
  lReplace("-0","","cn")
  lReplace("_1","","cn")
  lReplace("_0","","cn")
  lReplace("E","","cn")
  lReplace("F","","cn")
  lReplace("S0","","cn")
  lReplace("S1","","cn")
  lReplace("T","","cn")
  lReplace("G","","cn")
  lReplace("H","","cn")
  lReplace("!","","cn")
  lReplace("!","","cn")
  lReplace("!","","cn")
  lReplace("!$","","cn")
  lReplace("!,","","cn")
  lReplace("W1","","cn")
  lReplace("W0","","cn")
  PopPosition()
  Down()
end tCancelLine

// New keys and menus **************************************************
forward proc tDisableNewKeys()
forward menu tMainMenu()
forward menu tCancelMenu()
forward menu tFontMenu()

// Add the new key definitions
keydef new_keys
  <CtrlAlt C>      tCancelMenu()
  <CtrlAlt F>      tFontMenu()
  <CtrlAlt L>      tCancelLine()
  <CtrlAlt 0>      tDisableNewKeys()
  <CtrlAlt H>      QuickHelp(tHelpData)
  <F11>            tMainMenu()
end

// Disabling the new extra keys ***************************************
proc tDisableNewKeys()
  if YesNo("Disable the extra keys:") == 1 Disable(new_keys) endif
end

//  ***********************************************
Menu tFontMenu()
  Title = "Fonts"
  Width = 19
  x = 40
  y = 14
  history
  "Elite &Normal"                  , tEliteNormal()
  "Elite &Condensed"               , tEliteCondensed()
  "Elite Condensed &Emphasized"    , tEliteCondensedEmphasized()
  "&Wide"                          , tWide()
  "W&ide Emphasized"               , tWideEmphasized()
  "Enlarged &On"                   , tEnlargedOn()
  "Enlarged O&ff"                  , tEnlargedOff()
end tFontMenu

//  ***********************************************
Menu tCancelMenu()
  Title = "Cancel sequences"
  Width = 19
  x = 40
  y = 17
  history
  "&Cancel all through the file"        , tCancelAll()
  "Cancel on current &line <CtrlAlt L>" , tCancelLine()
end tCancelMenu

//  ***********************************************
Menu tMainMenu()
  Title = "Insert Escape Sequences"
  Width = 19
  x = 5
  y = 3
  history
  "Reset"                            , tReset()
  "Underline on"                     , tUnderlineOn()
  "Underline off"                    , tUnderlineOff()
  "Overscore on"                     , tOverscoreOn()
  "Overscore off"                    , tOverscoreOff()
  "Double-Strike Mode on"            , tDoubleStrikeOn()
  "Double-Strike Mode off"           , tDoubleStrikeOff()
  "Emphasized Mode on"               , tEmphasizedOn()
  "Emphasized Mode off"              , tEmphasizedOff()
  "Superscript Mode On"              , tSuperScriptOn()
  "Subscript Mode On"                , tSubScriptOn()
  "Super/Subscript Mode Off"         , tSuperScriptOff()
  "&Fonts <CtrlAltF> "              , tFontMenu(),        DontClose
  "",,Divide
  "&Cancel esc seq <CtrlAltC> "     , tCancelMenu(),      DontClose
  "",,Divide
  "Disable &new keys  <CtrlAlt 0>"   , tDisableNewKeys()
  "&Help              <CtrlAlt H>"   , QuickHelp(tHelpData)
  "This Menu         <F11>"          , tMainMenu()
end tMainMenu

proc Main()
  Enable (new_keys)
  tMainMenu()
end
